#!/bin/sh

. /lib/security-functions.sh

mount -o remount,rw "${root_device}" "${NEWROOT}"

set_3adm_env()
{
	[ -z "$1" ] && echo 0

	if [ x"$1" = x"0" ]; then
		user="root"
		uid="0"
	elif [ x"$1" = x"1" ]; then
		user="secadm"
		uid="600"
		auduid="700"
	fi

	chown -R "$uid":"$uid" ${NEWROOT}/etc/selinux
	chown -R "$uid":"$uid" ${NEWROOT}/usr/share/selinux
	chown -R "$uid":"$uid" ${NEWROOT}/var/lib/sepolgen
	chown -R "$uid":"$uid" ${NEWROOT}/var/lib/selinux
	if [ x"$auduid" = x"700" ]; then
		chown -R "$auduid":"$auduid" ${NEWROOT}/etc/audi*
		chown -R "$auduid":"$auduid" ${NEWROOT}/var/log/audit
	else
		chown -R "$uid":"$uid" ${NEWROOT}/etc/audi*
		chown -R "$uid":"$uid" ${NEWROOT}/var/log/audit
	fi

	if [ x"$uid" = x"0" ]; then
		setfacl -R -b ${NEWROOT}/var/log/
	else
		setfacl -R -m u:$auduid:r-x ${NEWROOT}/var/log/
	fi

	KYSEC_DB_FILE=${NEWROOT}/etc/kysec/db/whlist.db
	KYSEC_DEBUS_SERVICE=${NEWROOT}/usr/share/dbus-1/system-services/com.kylin.kysec.service
	KYSEC_SYNC_SERVICE=${NEWROOT}/lib/systemd/system/kysec-sync-notify.service

	[ -f "$KYSEC_DB_FILE" ] && \
		chown "$uid":"$uid" $KYSEC_DB_FILE

	if [ -f "$KYSEC_DEBUS_SERVICE" ]; then
		sed -i /^User=/cUser=$user $KYSEC_DEBUS_SERVICE
		setfattr -n security.kysec -v none:none:original $KYSEC_DEBUS_SERVICE
	fi
	if [ -f "$KYSEC_SYNC_SERVICE" ]; then
		sed -i /^User=/cUser=$user $KYSEC_SYNC_SERVICE
		setfattr -n security.kysec -v none:none:original $KYSEC_SYNC_SERVICE
	fi
}

if [ -f "$sysfs_entry" ]; then
	if [ -f "/.3adm" ]; then
		# enable 3adm
		adm_status=$(cat /.3adm)
		set_3adm_env "$adm_status"
		echo "$adm_status" > "$sysfs_entry"
		rm -rf /.3adm
		info "kysec 3adm enabled"
	else
		adm_status="0"
		set_3adm_env "$adm_status"
	fi
else
	set_3adm_env "0"
fi
