#!/bin/sh

. /lib/security-functions.sh

clear

if [ 0 -eq "$(kysec_is_enabled)" ]; then
	return 0
fi

kysec_whlist_db_dir=/etc/kysec/db
kysec_whlist_db=/etc/kysec/db/whlist.db
kysec_whlist_db_back=/etc/kysec/db/whlist_back.db
kysec_init=/usr/sbin/kysec_init

sm3_and_kmodpro_init()
{
	db_file="$1"

	if [ -e ${NEWROOT}/usr/bin/sm3-whitelist ]; then
		sqlite3 "$db_file" ".mod tabs" ".output ${NEWROOT}/etc/whitelist/sm3" "select hash,path from objects_list order by id asc;"
	fi

	kmod_protect_list=${NEWROOT}/usr/share/kylin-security-daemon/data/kmod-protect.list
	if [ -f "$kmod_protect_list"  ]; then
		cat $kmod_protect_list | while read path
		do
			[ -z "$path" ] && continue
			real_path=$(realpath ${NEWROOT}/$path 2>/dev/null)
			for i in "$real_path"
			do
				if [ -e "$i" ]; then
					f=${i#*"${NEWROOT}"}
					sqlite3 "$db_file" "update objects_list set rmmodpro=1 where path='"$f"';"
				fi
			done
		done
	fi
}

relabel_system_for_whlist()
{
	i=0
	j=0
	paths=

	cp "${NEWROOT}/$kysec_whlist_db" "${NEWROOT}/$kysec_whlist_db_back"
	sqlite3 "${NEWROOT}/$kysec_whlist_db_back" 'delete from objects_list;'
	for path in $(ls -a ${NEWROOT})
	do
		if [ $path != "." -a $path != ".." -a \
		     $path != "proc" -a $path != "run" -a \
		     $path != "sys" -a $path != "dev" -a \
		     $path != "tmp" -a $path != "cdrom" -a \
		     $path != "mnt" -a $path != "media" -a \
		     ! -h "${NEWROOT}/$path" ]; then
			if [ -d "${NEWROOT}/$path" ]; then
				if [ $path = "usr" -o $path = "home" ]; then
					for f in $(ls -a "${NEWROOT}/$path")
					do
						if [ $f != "." -a $f != ".." -a \
						     ! -h "${NEWROOT}/$path/$f" ]; then
							eval a$i="${NEWROOT}/$path/$f"
							i=$(($i+1))
						fi
					done
				else
					eval a$i="${NEWROOT}/$path"
					i=$(($i+1))
				fi
			elif [ -f "${NEWROOT}/$i" ]; then
				setfattr -n "security.kysec" -v "none:none:original" "${NEWROOT}/$path"
			fi
		fi
	done

	while [ $j -le $i ]
	do
		paths=$(eval echo "$paths \$a$j")
		j=$(($j+1))
	done

	show_message "正在扫描文件系统..."
	ls -d $paths | xargs -i -P $(nproc) \
		$kysec_init --rootmnt ${NEWROOT} -o ${NEWROOT}/tmp --scan {}

	rm -f ${NEWROOT}/tmp/list_box ${NEWROOT}/tmp/list_backup

	show_message "正在初始化白名单..."
	$kysec_init --rootmnt ${NEWROOT} --whlist ${NEWROOT}/tmp/list_*

	sm3_and_kmodpro_init "${NEWROOT}/$kysec_whlist_db_back"

	mv "${NEWROOT}/$kysec_whlist_db_back" "${NEWROOT}/$kysec_whlist_db"
	rm -f ${NEWROOT}/tmp/list_*

	setfattr -n "security.kysec" -v "none:none:trusted" ${NEWROOT}
}

relabel_system_from_whlist()
{
	$kysec_init --label --rootmnt ${NEWROOT}
}

relabel_file=${NEWROOT}/.exectl
relabelall_file=${NEWROOT}/.exectl_all

do_remount()
{
	old_value=`cat $kysec_status_path`
	mount -o remount,rw ${root_device} ${NEWROOT}
	set_kysec_status 4

	mount -t proc -o nodev,noexec,nosuid proc "$NEWROOT/proc"
	mount -t sysfs -o nodev,noexec,nosuid sys "$NEWROOT/sys"
	mount -t devtmpfs udev "$NEWROOT/dev"

	chroot "${NEWROOT}" /bin/mount -a

	if [ "$1" = "fs" ]; then
		relabel_system_for_whlist
	elif [ "$1" = "db" ]; then
		relabel_system_from_whlist
	fi

	if [ -e $relabelall_file ]; then
		rm -f $relabelall_file
	elif [ -e $relabel_file ]; then
		rm -f $relabel_file
	fi
	set_kysec_status $old_value

	umount "$NEWROOT/dev"
	umount "$NEWROOT/sys"
	umount "$NEWROOT/proc"
}

export exectl=

for x in $(cat /proc/cmdline); do
	case $x in
	exectl=*)
		exectl=${x#exectl=}
		;;
	esac
done

relabel_flag=
if [ -f $relabelall_file ]; then
	relabel_flag="1"
elif [ -f $relabel_file ]; then
	relabel_flag="2"
elif [ -n "${exectl}" ]; then
	relabel_flag="${exectl}"
fi

if [ -n "$relabel_flag" ];then
	# disable kernel messages temporarily
	level=`cat /proc/sys/kernel/printk | awk '{print $1}'`
	show_message "文件系统Kysec安全标记需要重新初始化"
	show_message "正在初始化系统执行控制安全标记，请稍候..."
	echo 1 > /proc/sys/kernel/printk
	if [ "x$relabel_flag" = "x1" ]; then
		do_remount fs
	elif [ "x$relabel_flag" = "x2" ]; then
		do_remount db
	fi
	# enable kernel messages
	echo "$level" > /proc/sys/kernel/printk
	clear_message
fi

clear
