#!/bin/sh

. /lib/security-functions.sh

# Mount the selinux directory in both the ramdisk's root as well as in
# the real root directory.
mkdir -p /selinux

clear

# Get the selinux configuration variables
SELINUX="disabled"
SELINUXTYPE=""

[ -e "$NEWROOT/etc/selinux/config" ] && . "$NEWROOT/etc/selinux/config"

selinux_is_enable() {
	if [ "$SELINUX" != "disabled" -o \
	     -f "/sys/fs/selinux/enforce" ]; then
		echo 1
	else
		echo 0
	fi
}

mount_backup_partition() {
	if [ -f $NEWROOT/etc/.bootinfo ]; then
		. $NEWROOT/etc/.bootinfo
		if [ "$SNAPSHOT_ENABLED" = 1 ]; then
			if [ -e $NEWROOT/dev/disk/by-uuid/$RECOVERY_DEV_UUID ]; then
				mount -t ext4 $NEWROOT/dev/disk/by-uuid/$RECOVERY_DEV_UUID $NEWROOT/backup
			fi
		fi
	fi
}

umount_backup_partition() {
	if [ -f $NEWROOT/etc/.bootinfo ]; then
		. $NEWROOT/etc/.bootinfo
		if [ "$SNAPSHOT_ENABLED" = 1 ]; then
			umount $NEWROOT/backup
		fi
	fi
}

if [ 0 -eq $(selinux_is_enable) ]; then
	return 0
fi

if [ "x${SELINUXTYPE}" = "x" -o \
     ! -d ${NEWROOT}/etc/selinux/${SELINUXTYPE} -o \
     ! -e ${NEWROOT}/etc/selinux/${SELINUXTYPE}/policy/policy.31 ]; then
	info "No SELinux policy found"
	return 0
fi

#check whether cmdline selinux autorelabel
export autorelabel=

for x in $(cat /proc/cmdline); do
	case $x in
	autorelabel=*)
		autorelabel=${x#autorelabel=}
		;;
	esac
done

mount -t sysfs -o nodev,noexec,nosuid sysfs ${NEWROOT}/sys
mount -t proc -o nodev,noexec,nosuid proc ${NEWROOT}/proc
mount -t devtmpfs udev $NEWROOT/dev

mount -t securityfs | grep -q "securityfs"
if [ $? -ne 0 ]; then
	mount -t securityfs -o nodev,noexec,nosuid none ${NEWROOT}/sys/kernel/security/
fi

mount -o remount,rw ${root_device} ${NEWROOT}

# Temporarily pivot to the real root directory, loading the policy
# from that disk.  Normally this process will occur by init, but kinit
# is not compiled against libselinux.  Therefore use load_policy to
# perform the same initialization.
chroot ${NEWROOT} /sbin/load_policy -i
RET=$?

mount -t selinuxfs none /selinux || warn "Unable to mount /selinux"

if [ $RET -eq 3 ]; then
	warn "SELinux policy load failed"
elif [ $RET -ne 0 ]; then
	warn "SELinux policy load failed, continuing"
else
	info "SELinux policy was loaded"
fi

prompt="
Relabeling The SELinux Policy context :
********************************************************
* If you are not already running SELinux, then you can *
* safely ignore the following error message.           *
* SELinux ${SELINUXTYPE} policy relabel is required.   *
* Relabeling could take a very long time, depending    *
* on file system size and speed of hard drives.	       *
********************************************************
* Relabeling files >>>>>>>>"

#/sbin/restorecon $(awk '!/^#/ && $4 !~ /noauto/ && $2 ~ /^\// { print $2 }' ${NEWROOT}/etc/fstab) >/dev/null 2>&1

statusfile=${NEWROOT}/.autorelabel

if [ $RET -eq 0 ]; then
	if [ -e $statusfile -o  "x${autorelabel}" = "xy" ]; then
		echo "$prompt"
		show_message "正在初始化强制访问控制标记，请稍后 ..."

		kysec_status=$(cat $kysec_status_path)
		if [ "$kysec_status" -ne 0 ]; then
			set_kysec_status 4
		fi

		se_status=$(cat /selinux/enforce)
		if [ "$se_status" -ne 0 ]; then
			echo "0" > /selinux/enforce
		fi

		if [ ! -e ${NEWROOT}/usr/share/selinux/${SELINUXTYPE}/.home ]; then
			chroot ${NEWROOT} /usr/sbin/genhomedircon
			touch ${NEWROOT}/usr/share/selinux/${SELINUXTYPE}/.home
		fi

		if [ -f ${NEWROOT}/usr/sbin/lvm ];then
			chroot "${NEWROOT}" /usr/sbin/lvm vgchange -ay
		fi

		chroot ${NEWROOT} /bin/mount -a
		mount_backup_partition
		chroot ${NEWROOT} /sbin/restorecon -RF / >/dev/null  2>&1
		umount_backup_partition
		chroot ${NEWROOT} /sbin/restorecon -RF /backup >/dev/null  2>&1
		rm -f $statusfile
		echo $se_status > /selinux/enforce
		set_kysec_status $kysec_status
	fi
fi

if [ -f "$sysfs_entry" -a  -f "/.3adm" ]; then
	chown -R 600:600 /selinux
fi

#relabel end#
umount ${NEWROOT}/proc
umount ${NEWROOT}/sys/fs/selinux
umount ${NEWROOT}/sys
umount $NEWROOT/dev
